<?php

/*
 * This file is part of the Comptoir-du-Libre software.
 * <https://gitlab.adullact.net/Comptoir/comptoir-du-libre>
 *
 * Copyright (c) ADULLACT   <https://adullact.org>
 *               Association des Développeurs et Utilisateurs de Logiciels Libres
 *               pour les Administrations et les Collectivités Territoriales
 *
 * Comptoir-du-Libre is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this software. If not, see <https://www.gnu.org/licenses/agpl-3.0.en.html>.
 */

declare(strict_types=1);

namespace App\Form\Organization;

use App\Entity\OrganizationType;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\ResetType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\CountryType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\UrlType;
use Symfony\Component\Form\Extension\Core\Type\FileType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Component\Translation\TranslatableMessage;
use Symfony\Component\Validator\Constraints\Length;
use Symfony\Component\Validator\Constraints\File;
use Symfony\Component\Validator\Constraints\NotBlank;
use Symfony\Component\Validator\Constraints\Url;
use Symfony\Component\Validator\Constraints\Hostname;

class OrganizationFormType extends AbstractType
{
    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $builder
            ->add(
                child: 'type',
                type: EntityType::class,
                options: array(
                    'label' => 'organization.form.label.org_type',
                    'help' => 'organization.form.label.org_type.help',
                    'class' => OrganizationType::class,
                    'choice_translation_domain' => true,
                    'choice_label' => function (?OrganizationType $organizationType): string {
                        $labelPrefix = 'organization.form.org_type.value.';
                        return $organizationType ? $labelPrefix . $organizationType->getName() : '';
                    },
                    'expanded' => true,
                )
            )
            ->add(
                child: 'countryName',
                type: CountryType::class,
                options: [
                //  'required' => false, // TODO FIXME
                    'mapped' => false,   // TODO FIXME
                    'attr' => [ // HTML attribut
                        'disabled' => true,
                    ],
                    'label' => 'organization.form.label.country',
                    'help' => 'organization.form.label.country.help',
                    'preferred_choices' => ['France', 'FR' ],
                //  'constraints' => [new NotBlank(['message' => 'common.validator.help.field.error.required'])],
                ]
            )
            ->add(
                child: 'name',
                type: TextType::class,
                options: [
                    'label' => 'organization.form.label.name',
                    'help' => 'organization.form.label.name.help',
                //  'sanitize_html' => true,
                //  'sanitizer' => 'user-data_remove_html',
                    'constraints' => [new NotBlank(['message' => 'common.validator.help.field.error.required'])],
                ]
            )
            ->add(
                child: 'siret',
                type: TextType::class,
                options: [
                    'required' => false,
                    'mapped' => false, // TODO FIXME
                    'attr' => [ // HTML attribut
                        'disabled' => true,
                    ],
                    'label' => 'organization.form.label.siret',
                    'help' => 'organization.form.label.siret.help',
                ]
            )
            ->add(
                child: 'website',
                type: UrlType::class,
                options: [
                    'required' => false,
                    'label' => 'organization.form.label.website',
                    'help' => 'organization.form.label.website.help',
                    'attr' => [ // HTML attribut
                        'placeholder' => 'https://example.org',
                    ],
                //  'sanitize_html' => true,
                //  'sanitizer' => 'user-data_remove_html',
                    'constraints' => [
                        // new Url(protocols: ['http', 'https'], relativeProtocol:  false, normalizer: 'strip_tags'),
                        new Url(protocols: ['http', 'https'], relativeProtocol:  false),
                    ],
                ]
            )
            ->add(
                child: 'logo',
                type: FileType::class,
                options: [
                    'mapped' => false,   // TODO FIXME
                    'required' => false,
                    'label' => 'organization.form.label.logo',
                    'help' => 'organization.form.label.logo.help',
                    'multiple' => false,
                    'attr' => [ // HTML attribut
                        'accept' => "image/png, image/jpeg",
                        'disabled' => true,
                    ],
                    'constraints' => [
                        new File([
                        //  'maxSize' => '1024k',
                            'maxSize' => '1m',
                            'mimeTypes' => [
                                'image/jpeg',
                                'image/png',
                            //  'image/gif',
                            ],
                            'mimeTypesMessage' => 'organization.form.label.logo.help.error.not_valid_file',
                        ])
                    ],
                ]
            )
            ->add(
                child: 'description',
                type: TextareaType::class,
                options:[
                    'required'   => false,
                    'mapped' => false,
                //  'sanitize_html' => true,
                //  'sanitizer' => 'user-data_remove_html',
                    'attr' => [ // HTML attribut
                        'rows' => 6,
                    ],
                    'data' => $options['organization_description'],
                    'label' => 'organization.form.label.description',
                    'help' => 'organization.form.label.description.help',
                ]
            )
            ->add('reset', ResetType::class, [
                'attr' => ['class' => 'save'],
              ]);
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'organization_description' => '',

            // enable/disable CSRF protection for this form
            'csrf_protection' => true,
            // the name of the hidden HTML field that stores the token
            'csrf_field_name' => 'csrf_token_ChangePassword',
            // an arbitrary string used to generate the value of the token
            // using a different string for each form improves its security
            'csrf_token_id'   => 'task_ChangePasswordFormType',
        ]);

        // you can also define the allowed types, allowed values and
        // any other feature supported by the OptionsResolver component
        $resolver->setAllowedTypes('organization_description', [ 'null', 'string']);
    }
}
